<?php

namespace Drupal\google_pubsub\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\google_pubsub\Services\PubSubServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form to confirm the deletion of a PubSub subscription.
 */
class DeleteSubscriptionConfirmForm extends ConfirmFormBase {

  /**
   * The PubSub service.
   *
   * @var \Drupal\google_pubsub\Services\PubSubServiceInterface
   */
  protected $pubSubService;

  /**
   * The Topic name.
   *
   * @var string
   */
  protected $topicName;

  /**
   * The Subscription name.
   *
   * @var string
   */
  protected $subscriptionName;

  /**
   * Constructs a new DeleteSubscriptionConfirmForm.
   *
   * @param \Drupal\google_pubsub\Services\PubSubServiceInterface $pubSubService
   *   The PubSub service.
   */
  public function __construct(PubSubServiceInterface $pubSubService) {
    $this->pubSubService = $pubSubService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('google_pubsub.pub_sub_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'google_pubsub_delete_subscription_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the subscription %subscription?', ['%subscription' => $this->subscriptionName]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    $request = $this->requestStack->getCurrentRequest();
    $referer = $request->headers->get('referer');
    if ($referer) {
      return Url::fromUri($referer);
    }
    else {
      return new Url('google_pubsub.topics');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('This action cannot be undone.');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $topicName = NULL, $subscriptionName = NULL) {
    $this->subscriptionName = $subscriptionName;
    $this->topicName = $topicName;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {

      $this->pubSubService->deleteSubscription($this->subscriptionName);
      $this->messenger()->addMessage($this->t('The subscription %subscription has been deleted.', ['%subscription' => $this->subscriptionName]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Failed to delete the subscription %subscription: %error', [
        '%subscription' => $this->subscriptionName,
        '%error' => $e->getMessage(),
      ]));
    }
    $form_state->setRedirect('google_pubsub.subscriptions', ['topicName' => $this->topicName]);
  }

}
