<?php

namespace Drupal\google_pubsub\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\google_pubsub\Services\PubSubServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form to update a PubSub topic.
 */
class UpdateTopicForm extends FormBase {

  /**
   * The PubSub service.
   *
   * @var \Drupal\google_pubsub\Services\PubSubServiceInterface
   */
  protected $pubSubService;

  /**
   * Constructs a new UpdateTopicForm.
   *
   * @param \Drupal\google_pubsub\Services\PubSubServiceInterface $pubSubService
   *   The PubSub service.
   */
  public function __construct(PubSubServiceInterface $pubSubService) {
    $this->pubSubService = $pubSubService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('google_pubsub.pub_sub_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'google_pubsub_update_topic_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $topicName = NULL) {
    $topic = $this->pubSubService->getTopic($topicName);
    $topic_info = $topic->info();

    $form['labels'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Labels'),
      '#default_value' => json_encode($topic_info['labels'] ?? [], JSON_PRETTY_PRINT),
      '#description' => $this->t('Enter labels as JSON. Example: {"key1": "value1", "key2": "value2"}'),
    ];

    $form['allowedPersistenceRegions'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed Persistence Regions'),
      '#default_value' => json_encode($topic_info['messageStoragePolicy']['allowedPersistenceRegions'] ?? [], JSON_PRETTY_PRINT),
      '#description' => $this->t('Enter allowed persistence regions as JSON array. Example: ["us-central1", "us-east1"]'),
    ];

    $form['schema'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Schema'),
      '#default_value' => $topic_info['schemaSettings']['schema'] ?? '',
      '#description' => $this->t('Enter the schema name. Example: projects/my-project/schemas/my-schema'),
    ];

    $form['encoding'] = [
      '#type' => 'select',
      '#title' => $this->t('Encoding'),
      '#options' => [
        'ENCODING_UNSPECIFIED' => $this->t('Unspecified'),
        'JSON' => $this->t('JSON'),
        'BINARY' => $this->t('Binary'),
      ],
      '#default_value' => $topic_info['schemaSettings']['encoding'] ?? 'ENCODING_UNSPECIFIED',
    ];

    $form['messageRetentionDuration'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Message Retention Duration'),
      '#default_value' => $topic_info['messageRetentionDuration'] ?? '',
      '#description' => $this->t('Enter the message retention duration in seconds. Example: 600s'),
    ];

    $form['kmsKeyName'] = [
      '#type' => 'textfield',
      '#title' => $this->t('KMS Key Name'),
      '#default_value' => $topic_info['kmsKeyName'] ?? '',
      '#description' => $this->t('Enter the KMS Key name. Example: projects/my-project/locations/global/keyRings/my-key-ring/cryptoKeys/my-key'),
    ];

    $form['satisfiesPzs'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Satisfies PZS'),
      '#default_value' => $topic_info['satisfiesPzs'] ?? FALSE,
      '#description' => $this->t('Indicates if the topic satisfies PZS (Protected Zone Storage).'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update Topic'),
    ];

    $form['topic_name'] = [
      '#type' => 'value',
      '#value' => $topicName,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $labels = $form_state->getValue('labels');
    $allowedPersistenceRegions = $form_state->getValue('allowedPersistenceRegions');

    if (!empty($labels) && !json_decode($labels)) {
      $form_state->setErrorByName('labels', $this->t('Labels must be a valid JSON object.'));
    }

    if (!empty($allowedPersistenceRegions) && !json_decode($allowedPersistenceRegions)) {
      $form_state->setErrorByName('allowedPersistenceRegions', $this->t('Allowed Persistence Regions must be a valid JSON array.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $topicName = $form_state->getValue('topic_name');
    $labels = json_decode($form_state->getValue('labels'), TRUE);
    $allowedPersistenceRegions = json_decode($form_state->getValue('allowedPersistenceRegions'), TRUE);
    $schema = $form_state->getValue('schema');
    $encoding = $form_state->getValue('encoding');
    $messageRetentionDuration = $form_state->getValue('messageRetentionDuration');
    $kmsKeyName = $form_state->getValue('kmsKeyName');
    $satisfiesPzs = $form_state->getValue('satisfiesPzs');

    $data = [
      'labels' => $labels,
      'messageStoragePolicy' => [
        'allowedPersistenceRegions' => $allowedPersistenceRegions,
      ],
      'schemaSettings' => [
        'schema' => $schema,
        'encoding' => $encoding,
      ],
      'messageRetentionDuration' => $messageRetentionDuration,
      'kmsKeyName' => $kmsKeyName,
      'satisfiesPzs' => $satisfiesPzs,
    ];

    try {
      $this->pubSubService->updateTopic($topicName, $data);
      $this->messenger()->addMessage($this->t('The topic %topic has been updated.', ['%topic' => $topicName]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Failed to update the topic %topic: %error', [
        '%topic' => $topicName,
        '%error' => $e->getMessage(),
      ]));
    }
  }

  /**
   * Provides the title for the update topics page.
   *
   * @return string
   *   The title of the page.
   */
  public function updateTopicTitle($topicName) {
    return $this->t('Update Google PubSub Topic: @topicName', ['@topicName' => urldecode($topicName)]);
  }

}
