<?php

namespace Drupal\google_pubsub\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Google\Cloud\PubSub\Message;
use Google\Cloud\PubSub\PubSubClient;
use Google\Cloud\PubSub\Subscription;
use Google\Cloud\PubSub\Topic;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Service for interacting with Google Cloud Pub/Sub.
 */
class PubSubService implements PubSubServiceInterface {
  use StringTranslationTrait;

  /**
   * The logger channel factory.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The serializer service.
   *
   * @var \Symfony\Component\Serializer\SerializerInterface
   */
  protected $serializer;

  /**
   * PubSubService constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   *   The serializer service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $loggerFactory,
    EntityTypeManagerInterface $entityTypeManager,
    ConfigFactoryInterface $configFactory,
    FileSystemInterface $fileSystem,
    SerializerInterface $serializer,
  ) {
    $this->logger = $loggerFactory->get('google_pubsub');
    $this->entityTypeManager = $entityTypeManager;
    $this->config = $configFactory->get('google_pubsub.settings');
    $this->fileSystem = $fileSystem;
    $this->serializer = $serializer;
  }

  /**
   * {@inheritDoc}
   */
  public function getClient(): ?PubSubClient {
    $fileId = $this->config->get('google_application_credentials');
    $projectId = $this->config->get('project_id');

    if (!$fileId || !$projectId) {
      $this->logger->error('Configuration is missing. Please configure the module settings.');
      return NULL;
    }

    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entityTypeManager->getStorage('file')->load($fileId);
    if ($file == NULL) {
      $this->logger->error('Failed to load the credentials file.');
      return NULL;
    }

    $credentials = $this->fileSystem->realpath($file->getFileUri());
    if ($credentials === FALSE) {
      $this->logger->error('Failed to get the real path of the credentials file.');
      return NULL;
    }

    putenv('GOOGLE_APPLICATION_CREDENTIALS=' . $credentials);

    $this->logger->info('Creating PubSub client with project ID: @projectId and credentials file: @credentials', [
      '@projectId' => $projectId,
      '@credentials' => $credentials,
    ]);

    try {
      $pubSubClient = new PubSubClient([
        'projectId' => $projectId,
      ]);
      return $pubSubClient;
    }
    catch (\Exception $e) {
      $this->logger->error('Error while creating the PubSub client: ' . $e->getMessage());
      return NULL;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function getTopic($topicName = NULL): ?Topic {
    if ($topicName == NULL) {
      $topicName = $this->config->get('topic_id');
      if (!$topicName) {
        $this->logger->error('Configuration is missing. Please configure the module settings.');
        return NULL;
      }
    }

    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return NULL;
    }

    return $pubSubClient->topic($topicName);
  }

  /**
   * {@inheritDoc}
   */
  public function publishMessage(string $message): void {
    $topic = $this->getTopic();
    if (!$topic) {
      throw new \Exception('Failed to get the topic.');
    }
    $data = [
      'data' => $message,
    ];
    try {
      $topic->publish($data);
      $this->logger->notice($this->t('Message published to @topic successfully.', ['@topic' => $topic->name()]));
    }
    catch (\Exception $e) {
      $this->logger->error("Error while publishing to pubsub: " . $e->getMessage());
      throw new \Exception("Error while publishing to pubsub: " . $e->getMessage());
    }
  }

  /**
   * {@inheritDoc}
   */
  public function createTopic(string $topicName): ?Topic {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return NULL;
    }
    try {
      $topic = $pubSubClient->createTopic($topicName);
      $this->logger->notice($this->t('Topic @topic created successfully.', ['@topic' => $topicName]));
      return $topic;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while creating topic: " . $topicName . ". " . $e->getMessage());
      return NULL;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function deleteTopic(string $topicName): bool {
    $topic = $this->getTopic($topicName);
    if (!$topic) {
      return FALSE;
    }
    try {
      $topic->delete();
      $this->logger->notice($this->t('Topic @topic deleted successfully.', ['@topic' => $topicName]));
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while deleting topic: " . $topicName . ". " . $e->getMessage());
      return FALSE;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function updateTopic(string $topicName, array $data): void {
    $topic = $this->getTopic($topicName);
    if (!$topic) {
      throw new \Exception("Topic $topicName not found.");
    }

    $updateMask = [];
    if (isset($data['labels'])) {
      $updateMask[] = 'labels';
    }
    if (isset($data['messageStoragePolicy']['allowedPersistenceRegions'])) {
      $updateMask[] = 'messageStoragePolicy.allowedPersistenceRegions';
    }
    if (isset($data['schemaSettings'])) {
      $updateMask[] = 'schemaSettings.schema';
      $updateMask[] = 'schemaSettings.encoding';
    }
    if (isset($data['messageRetentionDuration'])) {
      $updateMask[] = 'messageRetentionDuration';
    }
    if (isset($data['kmsKeyName'])) {
      $updateMask[] = 'kmsKeyName';
    }
    if (isset($data['satisfiesPzs'])) {
      $updateMask[] = 'satisfiesPzs';
    }

    try {
      $topic->update($data, ['updateMask' => $updateMask]);
      $this->logger->info("Updated topic $topicName successfully.");
    }
    catch (\Exception $e) {
      $this->logger->error("Error updating topic $topicName: " . $e->getMessage());
      throw $e;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function listTopics(): array {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return [];
    }
    try {
      $topics = $pubSubClient->topics();
      $this->logger->notice($this->t('Listed topics successfully.'));
      return iterator_to_array($topics);
    }
    catch (\Exception $e) {
      $this->logger->error("Error while listing topics: " . $e->getMessage());
      return [];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function createSubscription(string $subscriptionName, string $topicName): ?Subscription {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return NULL;
    }
    try {
      $topic = $pubSubClient->topic($topicName);
      $subscription = $topic->subscribe($subscriptionName);
      $this->logger->notice($this->t('Subscription @subscription created successfully for topic @topic.', [
        '@subscription' => $subscriptionName,
        '@topic' => $topicName,
      ]));
      return $subscription;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while creating subscription: " . $e->getMessage());
      return NULL;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function deleteSubscription(string $subscriptionName): bool {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return FALSE;
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $subscription->delete();
      $this->logger->notice($this->t('Subscription @subscription deleted successfully.', ['@subscription' => $subscriptionName]));
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while deleting subscription: " . $e->getMessage());
      return FALSE;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function listSubscriptions(string $topicName): array {
    $topic = $this->getTopic();
    if (!$topic) {
      return [];
    }
    try {
      $subscriptions = $topic->subscriptions();
      $this->logger->notice($this->t('Listed subscriptions successfully for topic @topic.', ['@topic' => $topicName]));
      return iterator_to_array($subscriptions);
    }
    catch (\Exception $e) {
      $this->logger->error("Error while listing subscriptions for topic: " . $topicName . ". " . $e->getMessage());
      return [];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function pullMessages(string $subscriptionName, int $maxMessages = 10): array {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return [];
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $messages = $subscription->pull(['maxMessages' => $maxMessages]);
      $subscription->acknowledgeBatch($messages);
      $this->logger->notice($this->t('Pulled and acknowledged @count messages from subscription @subscription.', [
        '@count' => count($messages),
        '@subscription' => $subscriptionName,
      ]));
      return $messages;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while pulling messages from subscription: " . $subscriptionName . ". " . $e->getMessage());
      return [];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function acknowledgeMessage(Message $message, string $subscriptionName): void {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return;
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $subscription->acknowledge($message);
      $this->logger->notice($this->t('Acknowledged message from subscription @subscription.', ['@subscription' => $subscriptionName]));
    }
    catch (\Exception $e) {
      $this->logger->error("Error while acknowledging message from subscription: " . $subscriptionName . ". " . $e->getMessage());
    }
  }

  /**
   * {@inheritDoc}
   */
  public function modifyPushConfig(string $subscriptionName, string $pushEndpoint): bool {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return FALSE;
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $subscription->modifyPushConfig(['pushEndpoint' => $pushEndpoint]);
      $this->logger->notice($this->t('Modified push configuration for subscription @subscription to endpoint @endpoint.', [
        '@subscription' => $subscriptionName,
        '@endpoint' => $pushEndpoint,
      ]));
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while modifying push configuration for subscription: " . $subscriptionName . ". " . $e->getMessage());
      return FALSE;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function retryUnacknowledgedMessages(string $subscriptionName, int $maxMessages = 10): array {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return [];
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $messages = $subscription->pull(['maxMessages' => $maxMessages]);
      foreach ($messages as $message) {
        // Process the message again.
        // This is where you can add your custom logic to retry processing.
        $this->logger->notice($this->t('Retrying message: @message', ['@message' => $message->data()]));
      }
      return $messages;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while retrying messages from subscription: " . $subscriptionName . ". " . $e->getMessage());
      return [];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function getTopicStats(string $topicName): array {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return [];
    }
    try {
      $topic = $pubSubClient->topic($topicName);
      $info = $topic->info();
      $this->logger->notice($this->t('Fetched stats for topic @topic successfully.', ['@topic' => $topicName]));
      return $info;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while fetching stats for topic: " . $topicName . ". " . $e->getMessage());
      return [];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function getSubscriptionStats(string $subscriptionName): array {
    $pubSubClient = $this->getClient();
    if (!$pubSubClient) {
      return [];
    }
    try {
      $subscription = $pubSubClient->subscription($subscriptionName);
      $info = $subscription->info();
      $this->logger->notice($this->t('Fetched stats for subscription @subscription successfully.', ['@subscription' => $subscriptionName]));
      return $info;
    }
    catch (\Exception $e) {
      $this->logger->error("Error while fetching stats for subscription: " . $subscriptionName . ". " . $e->getMessage());
      return [];
    }
  }

}
