<?php

namespace Drupal\google_pubsub\Services;

use Google\Cloud\PubSub\Message;
use Google\Cloud\PubSub\PubSubClient;
use Google\Cloud\PubSub\Subscription;
use Google\Cloud\PubSub\Topic;

/**
 * Interface for the PubSub service.
 *
 * Defines methods for interacting with Google Cloud Pub/Sub.
 */
interface PubSubServiceInterface {

  /**
   * Retrieves the PubSub client.
   *
   * @return \Google\Cloud\PubSub\PubSubClient|null
   *   The PubSub client or NULL on failure.
   */
  public function getClient(): ?PubSubClient;

  /**
   * Retrieves the PubSub topic.
   *
   * @param string|null $topicName
   *   The name of the topic to retrieve or NULL to use default config given.
   *
   * @return \Google\Cloud\PubSub\Topic|null
   *   The PubSub topic or NULL on failure.
   */
  public function getTopic($topicName = NULL): ?Topic;

  /**
   * Publishes a message to Google Cloud Pub/Sub.
   *
   * @param string $message
   *   The message to be published.
   *
   * @throws \Exception
   *   Throws exception when publish fails.
   */
  public function publishMessage(string $message): void;

  /**
   * Creates a new topic.
   *
   * @param string $topicName
   *   The name of the topic to create.
   *
   * @return \Google\Cloud\PubSub\Topic|null
   *   The created topic or NULL on failure.
   */
  public function createTopic(string $topicName): ?Topic;

  /**
   * Deletes a topic.
   *
   * @param string $topicName
   *   The name of the topic to delete.
   *
   * @return bool
   *   TRUE if the topic was deleted successfully, FALSE otherwise.
   */
  public function deleteTopic(string $topicName): bool;

  /**
   * Updates a PubSub topic.
   *
   * @param string $topicName
   *   The name of the topic.
   * @param array $data
   *   The data to update.
   *
   * @throws \Exception
   *   Throws exception when update fails.
   */
  public function updateTopic(string $topicName, array $data): void;

  /**
   * Lists all topics.
   *
   * @return \Google\Cloud\PubSub\Topic[]
   *   An array of topics.
   */
  public function listTopics(): array;

  /**
   * Creates a new subscription to a topic.
   *
   * @param string $subscriptionName
   *   The name of the subscription to create.
   * @param string $topicName
   *   The name of the topic to subscribe to.
   *
   * @return \Google\Cloud\PubSub\Subscription|null
   *   The created subscription or NULL on failure.
   */
  public function createSubscription(string $subscriptionName, string $topicName): ?Subscription;

  /**
   * Deletes a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription to delete.
   *
   * @return bool
   *   TRUE if the subscription was deleted successfully, FALSE otherwise.
   */
  public function deleteSubscription(string $subscriptionName): bool;

  /**
   * Lists all subscriptions for a topic.
   *
   * @param string $topicName
   *   The name of the topic to list subscriptions for.
   *
   * @return \Google\Cloud\PubSub\Subscription[]
   *   An array of subscriptions.
   */
  public function listSubscriptions(string $topicName): array;

  /**
   * Pulls messages from a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription to pull messages from.
   * @param int $maxMessages
   *   The maximum number of messages to pull.
   *
   * @return \Google\Cloud\PubSub\Message[]
   *   An array of messages.
   */
  public function pullMessages(string $subscriptionName, int $maxMessages = 10): array;

  /**
   * Acknowledges a message.
   *
   * @param \Google\Cloud\PubSub\Message $message
   *   The message to acknowledge.
   * @param string $subscriptionName
   *   The name of the subscription.
   */
  public function acknowledgeMessage(Message $message, string $subscriptionName): void;

  /**
   * Modifies the push configuration of a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription.
   * @param string $pushEndpoint
   *   The endpoint to push messages to.
   *
   * @return bool
   *   TRUE if the configuration was modified successfully, FALSE otherwise.
   */
  public function modifyPushConfig(string $subscriptionName, string $pushEndpoint): bool;

  /**
   * Retries unacknowledged messages in a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription to retry messages for.
   * @param int $maxMessages
   *   The maximum number of messages to pull and retry.
   *
   * @return \Google\Cloud\PubSub\Message[]
   *   An array of retried messages.
   */
  public function retryUnacknowledgedMessages(string $subscriptionName, int $maxMessages = 10): array;

  /**
   * Gets statistics of a topic.
   *
   * @param string $topicName
   *   The name of the topic.
   *
   * @return array
   *   An array of statistics for the topic.
   */
  public function getTopicStats(string $topicName): array;

  /**
   * Gets statistics of a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription.
   *
   * @return array
   *   An array of statistics for the subscription.
   */
  public function getSubscriptionStats(string $subscriptionName): array;

}
